﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Problems.VehicleRouting.Interfaces;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Problems.VehicleRouting.Variants;
using HeuristicLab.Problems.VehicleRouting.Encodings;

namespace HeuristicLab.Problems.VehicleRouting.ProblemInstances {
  [Item("CVRPEvaluator", "Represents a single depot CVRP evaluator.")]
  [StorableClass]
  public class CVRPEvaluator: SingleDepotVRPEvaluator {
    public ILookupParameter<DoubleValue> OverloadParameter {
      get { return (ILookupParameter<DoubleValue>)Parameters["Overload"]; }
    }

    protected override VRPEvaluation CreateTourEvaluation() {
      return new CVRPEvaluation();
    }

    protected override void EvaluateTour(VRPEvaluation eval, IVRPProblemInstance instance, Tour tour) {
      base.EvaluateTour(eval, instance, tour);

      ICapacitatedProblemInstance cvrpInstance = instance as ICapacitatedProblemInstance;

      double delivered = 0.0;
      double overweight = 0.0;

      for (int i = 0; i < tour.Stops.Count; i++) {
        delivered += instance.Demand[i];
      }

      double capacity = (instance as ICapacitatedProblemInstance).Capacity.Value;
      if (delivered > capacity) {
        overweight = delivered - capacity;
      }

      (eval as CVRPEvaluation).Overload = overweight;
      double penalty = overweight * cvrpInstance.OverloadPenalty.Value;
      eval.Penalty += penalty;
      eval.Quality += penalty;
    }

    protected override void InitResultParameters() {
      base.InitResultParameters();

      OverloadParameter.ActualValue = new DoubleValue(0);
    }

    protected override void UpdateResultParameters(VRPEvaluation tourEvaluation) {
      base.UpdateResultParameters(tourEvaluation);

      OverloadParameter.ActualValue.Value += (tourEvaluation as CVRPEvaluation).Overload;
    }
    
    [StorableConstructor]
    protected CVRPEvaluator(bool deserializing) : base(deserializing) { }

    public CVRPEvaluator() {
      Parameters.Add(new LookupParameter<DoubleValue>("Overload", "The overload."));
    }
  }
}